from pandas import read_table
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import tensorflow as tf
import keras
from keras.models import Sequential
from keras.layers import Dense, Dropout

def model_build(n_features, nodes,layers):
    model = tf.keras.models.Sequential()
    model.add(tf.keras.Input(shape=(n_features,)))
    
    for i in range(1, layers+1):
        model.add(tf.keras.layers.Dense(nodes,  activation='relu', kernel_initializer='he_normal'))
    
  

    model.add(tf.keras.layers.Dropout(0.2))
    model.add(tf.keras.layers.Dense(1, kernel_initializer='he_normal', activation='linear'))
    model.compile(optimizer=tf.keras.optimizers.Adam(learning_rate=1e-3), loss='mse', metrics = ['mse'])


    return model



def model_fit(b_size, n_epochs,model):
    model.fit(X_train, y_train, batch_size=b_size, epochs=n_epochs, verbose=0)
    return model

def score_calculate(y_pred, y_test):
    from sklearn.metrics import r2_score
    from sklearn.metrics import mean_absolute_error
    from sklearn.metrics import mean_squared_error
    r_2 = r2_score(y_test, y_pred)
    mae=mean_absolute_error(y_test,y_pred)
    mse=mean_squared_error(y_test,y_pred,squared=True)
    return r_2, mse, mae



filename='Simulation_generated_production_line_ASF_FDF.csv'

data= pd.read_csv(filename, sep=',',header=0, skipinitialspace=True,index_col=None)


print(data.head())


data=data.iloc[:, 1:] # to ignore job id column 


print(data.head())

arr = np.array(data, dtype=np.float)
    
#arr=arr[arr[:,0]==1]

    # Use the last column as the target value
X, y = arr[:, :-1], arr[:, -1]

frame_X=data.loc[:, data.columns != 'LeadTime']
f_names=list(frame_X.columns)

from sklearn.preprocessing import StandardScaler
scaler = StandardScaler()
scaler.fit(X)
X = scaler.transform(X)



print(X[0])
print(y[0])

seed=17
n_fold=10
from sklearn.model_selection import KFold
kfold = KFold(n_splits=n_fold, shuffle=True, random_state=seed)

iter=0


file_name= filename+ 'results.csv'
f = open(file_name, 'w')

f.write('Method No, FoldNo, R2, RMSE, MAE \n'.format(type))

for train, test in kfold.split(X, y):
    print ("Running Fold {}".format(iter))

   
    X_train, X_test, y_train, y_test = X[train], X[test], y[train], y[test]

        
    n_features = X_train.shape[1]

    from sklearn.feature_selection import SelectKBest, f_regression,mutual_info_regression
    fs = SelectKBest(score_func=f_regression, k=n_features)
# learn relationship from training data
    fs.fit(X_train, y_train)
    
    for i in range(len(fs.scores_)):
	    print('%s: %f' % (f_names[i], fs.scores_[i]))
  

         
 
    importances = fs.scores_
    feature_names=f_names

    #fig = plt.figure(figsize =(8, 7))
    #plt.rc('text', usetex=True)
    #plt.rc('font', weight='bold')
    #plt.rc('lines', lw=1, color='k')

    #fig=(pd.Series(importances, feature_names).nlargest(20).plot(kind='barh')) 
    #fig.plot()
    #plt.xticks(fontsize=20)
    #plt.yticks(fontsize=20)
    #plt.show()
    

    from sklearn.linear_model import Ridge
    clf = Ridge(alpha=1.0)
    clf.fit(X_train, y_train)
    y_pred=clf.predict(X_test)
    r_2, mse, mae=score_calculate(y_pred, y_test)


      
    print('Ridge model: r^2: %.3f, rmse: %.3f, mae: %.3f' % (r_2, np.sqrt(mse), mae))
    f.write('1, %f, %.3f, %.3f, %.3f \n' % (iter, r_2, np.sqrt(mse), mae))

    
    from sklearn.neighbors import KNeighborsRegressor
    neigh = KNeighborsRegressor(n_neighbors=10)
    neigh.fit(X_train, y_train)
    y_pred=neigh.predict(X_test)
    r_2, mse, mae=score_calculate(y_pred, y_test)
    print('k-NN model: r^2: %.3f, rmse: %.3f, mae: %.3f' % (r_2, np.sqrt(mse), mae))
    f.write('2, %f, %.3f, %.3f, %.3f \n' % (iter, r_2, np.sqrt(mse), mae))

    from sklearn.ensemble import RandomForestRegressor
    rf = RandomForestRegressor(n_estimators=300, min_samples_leaf=5, criterion='mse', max_features='sqrt')
    rf.fit(X_train, y_train)  
    y_pred=rf.predict(X_test)
    r_2, mse, mae=score_calculate(y_pred, y_test)
    print('RF model: r^2: %.3f, rmse: %.3f, mae: %.3f' % ( r_2, np.sqrt(mse), mae))
    f.write('3, %f, %.3f, %.3f, %.3f \n' % (iter, r_2, np.sqrt(mse), mae))
   

    model=model_build(n_features,100,5)
            
    model.compile(optimizer=tf.keras.optimizers.Adam(learning_rate=1e-3), loss='mse', metrics = ['mse'])


    model=model_fit(50, 100, model)
    y_pred=model.predict(X_test)
    r_2, mse, mae=score_calculate(y_pred, y_test)
           
    print('%.3f, %.3f, %.3f\n' % (r_2, np.sqrt(mse), mae))
    f.write('4, %f, %.3f, %.3f, %.3f \n' % (iter, r_2, np.sqrt(mse), mae))
    
    iter=iter+1

f.close()